@@ -50,7 +50,7 @@ context("code runs at all")#FOLDUP
 utils::data("cocktails", package="cocktailApp")
 
 test_that("shiny bits",{#FOLDUP
-	indat <- head(cocktails,100)
+	indat <- cocktails
 
 	expect_error(recipe_df <- .add_id(indat),NA)
 	expect_error(cocktail_df <- .distill_info(recipe_df),NA)
@@ -72,8 +72,62 @@ test_that("shiny bits",{#FOLDUP
 	expect_error(tbl <- .drinks_table(both6),NA)
 	expect_error(merged <- .merge_both(both6),NA)
 })#UNFOLD
+test_that("filter ingredients",{#FOLDUP
+	#indat <- head(cocktails,100)
+	indat <- cocktails
+
+	expect_error(recipe_df <- .add_id(indat),NA)
+	expect_error(cocktail_df <- .distill_info(recipe_df),NA)
+
+	both <- list(recipe=recipe_df %>% dplyr::select(-cocktail,-rating,-votes,-url),cocktail=cocktail_df)
+
+	expect_error(test0 <- .filter_ingredients(both,name_regex='',
+																						must_have_ing=c(),
+																						must_not_have_ing=c(),
+																						logical_sense='OR'),
+							 NA)
+	expect_error(test1 <- .filter_ingredients(both,name_regex='sazerac',
+																						must_have_ing=c('Bourbon','Averna'),
+																						must_not_have_ing=c(),
+																						logical_sense='OR'),
+							 NA)
+	expect_error(test2 <- .filter_ingredients(both,name_regex='sazerac',
+																						must_have_ing=c('Bourbon'),
+																						must_not_have_ing=c('Averna'),
+																						logical_sense='OR'),
+							 NA)
+	expect_error(test3 <- .filter_ingredients(both,name_regex='sazerac',
+																						must_have_ing=c('Bourbon'),
+																						must_not_have_ing=c('Averna'),
+																						logical_sense='AND'),
+							 NA)
+	expect_error(test4 <- .filter_ingredients(both,name_regex='',
+																						must_have_ing=c(),
+																						must_not_have_ing=c(),
+																						ing_regex='hartreus',
+																						logical_sense='OR'),
+							 NA)
+	expect_error(test5 <- .filter_ingredients(both,name_regex='',
+																						must_have_ing=c(),
+																						must_not_have_ing=c('Benedictine'),
+																						ing_regex='hartreus',
+																						logical_sense='OR'),
+							 NA)
+	expect_error(test6 <- .filter_ingredients(both,name_regex='sazerac',
+																						must_have_ing=c('Bourbon','Averna'),
+																						must_not_have_ing=c(),
+																						logical_sense='AND'),
+							 NA)
+	expect_error(test7 <- .filter_ingredients(both,name_regex='',
+																						must_have_ing=c('Bourbon'),
+																						must_not_have_ing=c(),
+																						logical_sense='AND',
+																						extra_ids=c(1,2,3,4)),
+							 NA)
+})#UNFOLD
 test_that('plot stuff',{# FOLDUP
-	indat <- head(cocktails,200)
+	#indat <- head(cocktails,200)
+	indat <- cocktails
 	expect_error(both <- .gen_both(indat),NA)
 	expect_error(both_alt <- .gen_both(),NA)
 
@@ -97,7 +151,8 @@ test_that('plot stuff',{# FOLDUP
 	expect_error(ph <- .make_bar_plot(merged),NA)
 })# UNFOLD
 test_that('correlation and coingredient',{# FOLDUP
-	indat <- head(cocktails,100)
+	#indat <- head(cocktails,100)
+	indat <- cocktails
 
 	#skip_on_cran()
 	#skip_on_travis()
